#! /usr/bin/env python
# -*- coding: utf-8 -*-
'''
Created on 2021/3/17 10:36:20

@author: LiuKuan
@copyright: Apache License, Version 2.0
'''
import random
#===============================================================================
# 
#===============================================================================
__all__ = ['rand_color']

_color_map = {}
_color_set = set()
_rand = random.Random()
_rand.seed(101)

def _add_color(color):
    _color_map[len(_color_map)] = color
    _color_set.add(color)

def _init_color_list():
    if not _color_map:
        _add_color((255, 0, 0))
        _add_color((0, 255, 0))
        _add_color((0, 0, 255))
        _add_color((255, 0, 255))
        _add_color((255, 255, 0))
        _add_color((0, 255, 255))
        _add_color((255, 250, 250))
        _add_color((255, 245, 238))
        _add_color((255, 228, 196))
        _add_color((255, 218, 185))
        _add_color((248, 248, 255))
        _add_color((245, 245, 245))
        _add_color((255, 222, 173))
        _add_color((255, 250, 205))
        _add_color((220, 220, 220))
        _add_color((255, 250, 240))
        _add_color((253, 245, 230))
        _add_color((250, 240, 230))
        _add_color((250, 235, 215))
        _add_color((255, 239, 213))
        _add_color((255, 235, 205))
        _add_color((255, 228, 181))
        _add_color((255, 248, 220))
        _add_color((255, 255, 240))
        _add_color((240, 255, 240))
        _add_color((245, 255, 250))
        _add_color((240, 255, 255))
        _add_color((240, 248, 255))
        _add_color((230, 230, 250))
        _add_color((255, 240, 245))
        _add_color((255, 228, 225))
        _add_color((47, 79, 79))
        _add_color((105, 105, 105))
        _add_color((112, 128, 144))
        _add_color((119, 136, 153))
        _add_color((190, 190, 190))
        _add_color((211, 211, 211))
        _add_color((25, 25, 112))
        _add_color((0, 0, 128))
        _add_color((100, 149, 237))
        _add_color((72, 61, 139))
        _add_color((106, 90, 205))
        _add_color((123, 104, 238))
        _add_color((132, 112, 255))
        _add_color((0, 0, 205))
        _add_color((65, 105, 225))
        _add_color((30, 144, 255))
        _add_color((0, 191, 255))
        _add_color((135, 206, 235))
        _add_color((135, 206, 250))
        _add_color((70, 130, 180))
        _add_color((176, 196, 222))
        _add_color((173, 216, 230))
        _add_color((176, 224, 230))
        _add_color((175, 238, 238))
        _add_color((0, 206, 209))
        _add_color((72, 209, 204))
        _add_color((64, 224, 208))
        _add_color((224, 255, 255))
        _add_color((95, 158, 160))
        _add_color((102, 205, 170))
        _add_color((127, 255, 212))
        _add_color((0, 100, 0))
        _add_color((85, 107, 47))
        _add_color((143, 188, 143))
        _add_color((46, 139, 87))
        _add_color((60, 179, 113))
        _add_color((32, 178, 170))
        _add_color((152, 251, 152))
        _add_color((0, 255, 127))
        _add_color((124, 252, 0))
        _add_color((127, 255, 0))
        _add_color((0, 250, 154))
        _add_color((173, 255, 47))
        _add_color((50, 205, 50))
        _add_color((154, 205, 50))
        _add_color((34, 139, 34))
        _add_color((107, 142, 35))
        _add_color((189, 183, 107))
        _add_color((238, 232, 170))
        _add_color((250, 250, 210))
        _add_color((255, 255, 224))
        _add_color((255, 215, 0))
        _add_color((238, 221, 130))
        _add_color((218, 165, 32))
        _add_color((184, 134, 11))
        _add_color((188, 143, 143))
        _add_color((205, 92, 92))
        _add_color((139, 69, 19))
        _add_color((160, 82, 45))
        _add_color((205, 133, 63))
        _add_color((222, 184, 135))
        _add_color((245, 245, 220))
        _add_color((245, 222, 179))
        _add_color((244, 164, 96))
        _add_color((210, 180, 140))
        _add_color((210, 105, 30))
        _add_color((178, 34, 34))
        _add_color((165, 42, 42))
        _add_color((233, 150, 122))
        _add_color((250, 128, 114))
        _add_color((255, 160, 122))
        _add_color((255, 165, 0))
        _add_color((255, 140, 0))
        _add_color((255, 127, 80))
        _add_color((240, 128, 128))
        _add_color((255, 99, 71))
        _add_color((255, 69, 0))
        _add_color((255, 105, 180))
        _add_color((255, 20, 147))
        _add_color((255, 192, 203))
        _add_color((255, 182, 193))
        _add_color((219, 112, 147))
        _add_color((176, 48, 96))
        _add_color((199, 21, 133))
        _add_color((208, 32, 144))
        _add_color((238, 130, 238))
        _add_color((221, 160, 221))
        _add_color((218, 112, 214))
        _add_color((186, 85, 211))
        _add_color((153, 50, 204))
        _add_color((148, 0, 211))
        _add_color((138, 43, 226))
        _add_color((160, 32, 240))
        _add_color((147, 112, 219))
        _add_color((216, 191, 216))
        _add_color((238, 233, 233))
        _add_color((205, 201, 201))
        _add_color((139, 137, 137))
        _add_color((238, 229, 222))
        _add_color((205, 197, 191))
        _add_color((139, 134, 130))
        _add_color((255, 239, 219))
        _add_color((238, 223, 204))
        _add_color((205, 192, 176))
        _add_color((139, 131, 120))
        _add_color((238, 213, 183))
        _add_color((205, 183, 158))
        _add_color((139, 125, 107))
        _add_color((238, 203, 173))
        _add_color((205, 175, 149))
        _add_color((139, 119, 101))
        _add_color((238, 207, 161))
        _add_color((205, 179, 139))
        _add_color((139, 121, 94))
        _add_color((238, 233, 191))
        _add_color((205, 201, 165))
        _add_color((139, 137, 112))
        _add_color((238, 232, 205))
        _add_color((205, 200, 177))
        _add_color((139, 136, 120))
        _add_color((238, 238, 224))
        _add_color((205, 205, 193))
        _add_color((139, 139, 131))
        _add_color((224, 238, 224))
        _add_color((193, 205, 193))
        _add_color((131, 139, 131))
        _add_color((238, 224, 229))
        _add_color((205, 193, 197))
        _add_color((139, 131, 134))
        _add_color((238, 213, 210))
        _add_color((205, 183, 181))
        _add_color((139, 125, 123))
        _add_color((224, 238, 238))
        _add_color((193, 205, 205))
        _add_color((131, 139, 139))
        _add_color((131, 111, 255))
        _add_color((122, 103, 238))
        _add_color((105, 89, 205))
        _add_color((71, 60, 139))
        _add_color((72, 118, 255))
        _add_color((67, 110, 238))
        _add_color((58, 95, 205))
        _add_color((39, 64, 139))
        _add_color((0, 0, 238))
        _add_color((0, 0, 139))
        _add_color((28, 134, 238))
        _add_color((24, 116, 205))
        _add_color((16, 78, 139))
        _add_color((99, 184, 255))
        _add_color((92, 172, 238))
        _add_color((79, 148, 205))
        _add_color((54, 100, 139))
        _add_color((0, 178, 238))
        _add_color((0, 154, 205))
        _add_color((0, 104, 139))
        _add_color((135, 206, 255))
        _add_color((126, 192, 238))
        _add_color((108, 166, 205))
        _add_color((74, 112, 139))
        _add_color((176, 226, 255))
        _add_color((164, 211, 238))
        _add_color((141, 182, 205))
        _add_color((96, 123, 139))
        _add_color((198, 226, 255))
        _add_color((185, 211, 238))
        _add_color((159, 182, 205))
        _add_color((108, 123, 139))
        _add_color((202, 225, 255))
        _add_color((188, 210, 238))
        _add_color((162, 181, 205))
        _add_color((110, 123, 139))
        _add_color((191, 239, 255))
        _add_color((178, 223, 238))
        _add_color((154, 192, 205))
        _add_color((104, 131, 139))
        _add_color((209, 238, 238))
        _add_color((180, 205, 205))
        _add_color((122, 139, 139))
        _add_color((187, 255, 255))
        _add_color((174, 238, 238))
        _add_color((150, 205, 205))
        _add_color((102, 139, 139))
        _add_color((152, 245, 255))
        _add_color((142, 229, 238))
        _add_color((122, 197, 205))
        _add_color((83, 134, 139))
        _add_color((0, 245, 255))
        _add_color((0, 229, 238))
        _add_color((0, 197, 205))
        _add_color((0, 134, 139))
        _add_color((0, 238, 238))
        _add_color((0, 205, 205))
        _add_color((0, 139, 139))
        _add_color((151, 255, 255))
        _add_color((141, 238, 238))
        _add_color((121, 205, 205))
        _add_color((82, 139, 139))
        _add_color((118, 238, 198))
        _add_color((69, 139, 116))
        _add_color((193, 255, 193))
        _add_color((180, 238, 180))
        _add_color((155, 205, 155))
        _add_color((105, 139, 105))
        _add_color((84, 255, 159))
        _add_color((78, 238, 148))
        _add_color((67, 205, 128))
        _add_color((154, 255, 154))
        _add_color((144, 238, 144))
        _add_color((124, 205, 124))
        _add_color((84, 139, 84))
        _add_color((0, 238, 118))
        _add_color((0, 205, 102))
        _add_color((0, 139, 69))
        _add_color((0, 238, 0))
        _add_color((0, 205, 0))
        _add_color((0, 139, 0))
        _add_color((118, 238, 0))
        _add_color((102, 205, 0))
        _add_color((69, 139, 0))
        _add_color((192, 255, 62))
        _add_color((179, 238, 58))
        _add_color((105, 139, 34))
        _add_color((202, 255, 112))
        _add_color((188, 238, 104))
        _add_color((162, 205, 90))
        _add_color((110, 139, 61))
        _add_color((255, 246, 143))
        _add_color((238, 230, 133))
        _add_color((205, 198, 115))
        _add_color((139, 134, 78))
        _add_color((255, 236, 139))
        _add_color((238, 220, 130))
        _add_color((205, 190, 112))
        _add_color((139, 129, 76))
        _add_color((238, 238, 209))
        _add_color((205, 205, 180))
        _add_color((139, 139, 122))
        _add_color((238, 238, 0))
        _add_color((205, 205, 0))
        _add_color((139, 139, 0))
        _add_color((238, 201, 0))
        _add_color((205, 173, 0))
        _add_color((139, 117, 0))
        _add_color((255, 193, 37))
        _add_color((238, 180, 34))
        _add_color((205, 155, 29))
        _add_color((139, 105, 20))
        _add_color((255, 185, 15))
        _add_color((238, 173, 14))
        _add_color((205, 149, 12))
        _add_color((139, 101, 8))
        _add_color((255, 193, 193))
        _add_color((238, 180, 180))
        _add_color((205, 155, 155))
        _add_color((139, 105, 105))
        _add_color((255, 106, 106))
        _add_color((238, 99, 99))
        _add_color((205, 85, 85))
        _add_color((139, 58, 58))
        _add_color((255, 130, 71))
        _add_color((238, 121, 66))
        _add_color((205, 104, 57))
        _add_color((139, 71, 38))
        _add_color((255, 211, 155))
        _add_color((238, 197, 145))
        _add_color((205, 170, 125))
        _add_color((139, 115, 85))
        _add_color((255, 231, 186))
        _add_color((238, 216, 174))
        _add_color((205, 186, 150))
        _add_color((139, 126, 102))
        _add_color((255, 165, 79))
        _add_color((238, 154, 73))
        _add_color((139, 90, 43))
        _add_color((255, 127, 36))
        _add_color((238, 118, 33))
        _add_color((205, 102, 29))
        _add_color((255, 48, 48))
        _add_color((238, 44, 44))
        _add_color((205, 38, 38))
        _add_color((139, 26, 26))
        _add_color((255, 64, 64))
        _add_color((238, 59, 59))
        _add_color((205, 51, 51))
        _add_color((139, 35, 35))
        _add_color((255, 140, 105))
        _add_color((238, 130, 98))
        _add_color((205, 112, 84))
        _add_color((139, 76, 57))
        _add_color((238, 149, 114))
        _add_color((205, 129, 98))
        _add_color((139, 87, 66))
        _add_color((238, 154, 0))
        _add_color((205, 133, 0))
        _add_color((139, 90, 0))
        _add_color((255, 127, 0))
        _add_color((238, 118, 0))
        _add_color((205, 102, 0))
        _add_color((139, 69, 0))
        _add_color((255, 114, 86))
        _add_color((238, 106, 80))
        _add_color((205, 91, 69))
        _add_color((139, 62, 47))
        _add_color((238, 92, 66))
        _add_color((205, 79, 57))
        _add_color((139, 54, 38))
        _add_color((238, 64, 0))
        _add_color((205, 55, 0))
        _add_color((139, 37, 0))
        _add_color((238, 0, 0))
        _add_color((205, 0, 0))
        _add_color((139, 0, 0))
        _add_color((238, 18, 137))
        _add_color((205, 16, 118))
        _add_color((139, 10, 80))
        _add_color((255, 110, 180))
        _add_color((238, 106, 167))
        _add_color((205, 96, 144))
        _add_color((139, 58, 98))
        _add_color((255, 181, 197))
        _add_color((238, 169, 184))
        _add_color((205, 145, 158))
        _add_color((139, 99, 108))
        _add_color((255, 174, 185))
        _add_color((238, 162, 173))
        _add_color((205, 140, 149))
        _add_color((139, 95, 101))
        _add_color((255, 130, 171))
        _add_color((238, 121, 159))
        _add_color((205, 104, 137))
        _add_color((139, 71, 93))
        _add_color((255, 52, 179))
        _add_color((238, 48, 167))
        _add_color((205, 41, 144))
        _add_color((139, 28, 98))
        _add_color((255, 62, 150))
        _add_color((238, 58, 140))
        _add_color((205, 50, 120))
        _add_color((139, 34, 82))
        _add_color((238, 0, 238))
        _add_color((205, 0, 205))
        _add_color((139, 0, 139))
        _add_color((255, 131, 250))
        _add_color((238, 122, 233))
        _add_color((205, 105, 201))
        _add_color((139, 71, 137))
        _add_color((255, 187, 255))
        _add_color((238, 174, 238))
        _add_color((205, 150, 205))
        _add_color((139, 102, 139))
        _add_color((224, 102, 255))
        _add_color((209, 95, 238))
        _add_color((180, 82, 205))
        _add_color((122, 55, 139))
        _add_color((191, 62, 255))
        _add_color((178, 58, 238))
        _add_color((154, 50, 205))
        _add_color((104, 34, 139))
        _add_color((155, 48, 255))
        _add_color((145, 44, 238))
        _add_color((125, 38, 205))
        _add_color((85, 26, 139))
        _add_color((171, 130, 255))
        _add_color((159, 121, 238))
        _add_color((137, 104, 205))
        _add_color((93, 71, 139))
        _add_color((255, 225, 255))
        _add_color((238, 210, 238))
        _add_color((205, 181, 205))
        _add_color((139, 123, 139))
        _add_color((28, 28, 28))
        _add_color((54, 54, 54))
        _add_color((79, 79, 79))
        _add_color((130, 130, 130))
        _add_color((156, 156, 156))
        _add_color((181, 181, 181))
        _add_color((207, 207, 207))
        _add_color((232, 232, 232))
        _add_color((169, 169, 169))

def _gen_color():
    while 1:
        r = _rand.randint(0,255)
        g = _rand.randint(0,255)
        b = _rand.randint(0,255)
        c = (r,g,b)
        if c in _color_set:
            continue
        return c

def rand_color(idx = None):
    '''
    :param idx:  color index, when None, rand a color
    :return:
    '''
    if idx is None:
        return _gen_color()
    else:
        _init_color_list()
        if idx in _color_map:
            return _color_map[idx]
        c = _gen_color()
        _add_color(c)
        return c

_init_color_list()
